@@ -1,3 +1,9 @@
+"""
+
+Data exploration
+
+"""
+
 import copy
 import dask.dataframe as dd
 import numpy as np
@@ -7,15 +13,10 @@ import pandas as pd
 from .anomaly_detector import AnomalyDetector
 from .data_import_export import DataImporter
 from .data_visualizer import DataVisualizer
-from .text_miner import TextMiner
 from .utils import Log, PERCENTILES, StatsUtils, EasyExploreUtils
 from dask.array import from_array
 from dask.distributed import Client
-from typing import Dict, List, Tuple
-
-# TODO:
-#  Correlation -> Partial + Final Heat Map
-#  Data Distribution: Color (overtime) + Annotations (continuous distribution parameter)
+from typing import Dict, List, Tuple, Union
 
 
 class DataExplorerException(Exception):
@@ -30,7 +31,7 @@ class DataExplorer:
     Class for data exploration
     """
     def __init__(self,
-                 df=None,
+                 df: Union[dd.DataFrame, pd.DataFrame],
                  file_path: str = '',
                  table_name: str = None,
                  target: str = None,
@@ -423,24 +424,7 @@ class DataExplorer:
             if _ar.shape[0] == 0:
                 Log(write=False).log('Partial correlation not possible because of high amount of missing values')
             else:
-                #_n_features = _ar.shape[1]
-                #_partial_cor: np.array = np.zeros((_n_features, _n_features), dtype=np.float)
-                #for i in range(_n_features):
-                #    _partial_cor[i, i] = 1
-                #    for j in range(i + 1, _n_features):
-                #        _idx = np.ones(_n_features, dtype=np.bool)
-                #        _idx[i] = False
-                #        _idx[j] = False
-                #        _beta_i = linalg.lstsq(_ar[:, _idx], _ar[:, j])[0]
-                #        _beta_j = linalg.lstsq(_ar[:, _idx], _ar[:, i])[0]
-                #        _res_j = _ar[:, j] - _ar[:, _idx].dot(_beta_i)
-                #        _res_i = _ar[:, i] - _ar[:, _idx].dot(_beta_j)
-                #        _corr = stats.pearsonr(_res_i, _res_j)[0]
-                #        _partial_cor[i, j] = _corr
-                #        _partial_cor[j, i] = _corr
-                #_cor_matrix: pd.DataFrame = pd.DataFrame(data=_partial_cor, columns=self.feature_types.get('continuous'), index=self.feature_types.get('continuous'))
-                _partial_cor_matrix = StatsUtils(data=self.df, features=self.feature_types.get('continuous')).correlation(meth='partial')
-                #print(_partial_cor_matrix)
+                _partial_cor_matrix = StatsUtils(data=self.df).correlation(meth='partial', features=self.feature_types.get('continuous'))
                 _cor['partial'].update(_partial_cor_matrix.to_dict())
                 _cor_plot.update({'Partial Correlation': dict(data=_partial_cor_matrix,
                                                               features=self.feature_types.get('continuous'),
@@ -543,7 +527,12 @@ class DataExplorer:
                                                                )
                                   })
         if continuous:
-            _continuous_features: List[str] = [conti for conti in self.feature_types.get('continuous') if conti in _features]
+            _continuous_features: List[str] = []
+            for conti in self.feature_types.get('continuous'):
+                if conti in _features:
+                    if str(self.df[conti].dtype).find('float') < 0:
+                        self.df[conti] = self.df[conti].astype(float)
+                    _continuous_features.append(conti)
             if len(_continuous_features) > 0:
                 _desc: dict = self.df[_continuous_features].describe(percentiles=PERCENTILES).compute()
                 #_norm: dict = StatsUtils(data=self.df, features=self.feature_types.get('continuous')).normality_test(alpha=0.05, meth='shapiro-wilk')
@@ -573,10 +562,20 @@ class DataExplorer:
             if len(self.feature_types.get('date')) == 0:
                 Log(write=False, level='error').log('No time feature found in data set')
             else:
-                _date_features: List[str] = [date for date in self.feature_types.get('date') if date in _features]
+                __continuous_features: List[str] = []
+                for conti in self.feature_types.get('continuous'):
+                    if conti in _features:
+                        if str(self.df[conti].dtype).find('float') < 0:
+                            self.df[conti] = self.df[conti].astype(float)
+                        __continuous_features.append(conti)
+                _date_features: List[str] = []
+                for date in self.feature_types.get('date'):
+                    if date in _features:
+                        if str(self.df[date].dtype).find('object') >= 0:
+                            self.df[date] = dd.from_array(x=pd.to_datetime(self.df[date].values.compute(), errors='coerce'))
+                        _date_features.append(date)
                 for ft in _date_features:
                     _distribution[ft] = self.df[ft].value_counts(sort=True, ascending=False, dropna=self.include_nan).compute()
-                __continuous_features: List[str] = [conti for conti in self.feature_types.get('continuous') if conti in _features]
                 if len(_date_features) > 0 and len(__continuous_features) > 0:
                     _subplots.update({'Distribution over Time': dict(data=self.df,
                                                                      features=__continuous_features,
@@ -715,27 +714,32 @@ class DataExplorer:
                                                                    kwargs=dict(values=[_df_all_data.sum().sum(),
                                                                                        (self.n_cases * len(_features)) - _df_all_data.sum().sum()
                                                                                        ],
-                                                                               labels=['Valid Data', 'Missing Data']
+                                                                               labels=['Valid Data', 'Missing Data'],
+                                                                               layout={}
                                                                                )
                                                                    ),
                                       'Missing Data Distribution Case-wise': dict(data=_df_nan_case_sum,
                                                                                   features=[],
                                                                                   plot_type='pie',
                                                                                   kwargs=dict(values=list(_df_nan_case_sum.loc[:, 'N'].values),
-                                                                                              labels=list(_df_nan_case_sum.loc[:, 'N'].index.values)
+                                                                                              labels=list(_df_nan_case_sum.loc[:, 'N'].index.values),
+                                                                                              layout={}
                                                                                               )
                                                                                   ),
                                       'Missing Data Distribution Features-wise': dict(data=_df_feature_mis,
                                                                                       features=[],
                                                                                       plot_type='pie',
                                                                                       kwargs=dict(values=list(_df_feature_mis.loc[:, 'N'].values),
-                                                                                                  labels=list(_df_feature_mis.loc[:, 'N'].index.values)
+                                                                                                  labels=list(_df_feature_mis.loc[:, 'N'].index.values),
+                                                                                                  layout={}
                                                                                                   )
                                                                                       ),
                                       'Sparsity of the features': dict(data=_df_feature_mis,
                                                                        features=[],
                                                                        plot_type='table',
-                                                                       kwargs=dict(index_title='Features with Missing Data')
+                                                                       kwargs=dict(index_title='Features with Missing Data',
+                                                                                   layout={}
+                                                                                   )
                                                                        )
                                       })
                     if nan_heat_map:
@@ -748,7 +752,9 @@ class DataExplorer:
                                                                                    colorbar=dict(title='Value Range',
                                                                                                  tickvals=['0', '1'],
                                                                                                  ticktext=['Valid',
-                                                                                                           'Missing'])
+                                                                                                           'Missing']
+                                                                                                 ),
+                                                                                   layout={}
                                                                                    )
                                                                        )
                                           })
@@ -766,7 +772,8 @@ class DataExplorer:
                         _label.append('variant')
                 _subplots.update({'Invariant Features': dict(data=pd.DataFrame(data=dict(features=self.features, invariant=_label)),
                                                              features=['invariant'],
-                                                             plot_type='pie'
+                                                             plot_type='pie',
+                                                             kwargs=dict(layout={})
                                                              )
                                   })
         if duplicate_features or duplicate_cases:
@@ -795,14 +802,14 @@ class DataExplorer:
                     _subplots.update({'Duplicate Cases': dict(data=pd.DataFrame(data=dict(duplicate_cases=_duplicate_cases)),
                                                               features=['duplicate_cases'],
                                                               plot_type='pie',
-                                                              interactive=True
+                                                              kwargs=dict(layout={})
                                                               )
                                       })
                 if duplicate_features:
                     _subplots.update({'Duplicate Features': dict(data=pd.DataFrame(data=dict(features=_features, duplicate_features=_duplicate_features)),
                                                                  features=['duplicate_features'],
                                                                  plot_type='pie',
-                                                                 interactive=True
+                                                                 kwargs=dict(layout={})
                                                                  )
                                       })
         for mis_feature in _data_health['sparsity']['features']:
@@ -840,13 +847,16 @@ class DataExplorer:
                                                                                         colorbar=dict(title='Value Range',
                                                                                                       tickvals=['0', '1', '2', '3'],
                                                                                                       ticktext=['Valid', 'Missing', 'Invariant', 'Duplicate']
-                                                                                                      )
+                                                                                                      ),
+                                                                                        layout={}
                                                                                         )
                                                                             ),
                               'Data Health Check Summary': dict(data=_summary,
                                                                 features=[],
                                                                 plot_type='table',
-                                                                kwargs=dict(index_title='Attributes of Data Health')
+                                                                kwargs=dict(index_title='Attributes of Data Health',
+                                                                            layout={}
+                                                                            )
                                                                 )
                               })
             DataVisualizer(title='Data Health Check',
@@ -858,7 +868,7 @@ class DataExplorer:
                            render=True if self.file_path is None else False,
                            file_path=self.file_path
                            ).run()
-        del _results_after_cleaning
+            del _results_after_cleaning
         return dict(cases=list(set(_cases)), features=list(set(__features)))
 
     def data_typing(self) -> dict:
@@ -1054,6 +1064,12 @@ class DataExplorer:
                 _plot_type: str = plot_type
             else:
                 _plot_type: str = 'density'
+            if str(self.df[val].dtype).find('object') >= 0:
+                self.df[val] = self.df[val].astype(float)
+            if str(self.df[lat].dtype).find('object') >= 0:
+                self.df[lat] = self.df[lat].astype(float)
+            if str(self.df[lon].dtype).find('object') >= 0:
+                self.df[lon] = self.df[lon].astype(float)
             _df: dd.DataFrame = self.df[[val, lat, lon]]
             _df = _df.loc[~_df[val].isnull(), :]
             _df = _df.loc[~_df[lat].isnull(), :]
@@ -1119,7 +1135,12 @@ class DataExplorer:
             _features = self.features
         _subplots: dict = {}
         _outlier: Dict[str, List[int]] = {}
-        _continuous_features: List[str] = [conti for conti in self.feature_types.get('continuous') if conti in _features]
+        _continuous_features: List[str] = []
+        for conti in self.feature_types.get('continuous'):
+            if conti in _features:
+                if str(self.df[conti].dtype).find('float') < 0:
+                    self.df[conti] = self.df[conti].astype(float)
+                _continuous_features.append(conti)
         if len(_continuous_features) > 0:
             if kind == 'uni':
                 _outlier.update({'uni': self._check_outliers()})
@@ -1244,82 +1265,3 @@ class DataExplorer:
         else:
             Log(write=False, level='info').log(msg='No continuous features found to run outlier detection')
         return _outlier
-
-    def text_analyzer(self,
-                      lang: str = None,
-                      lang_model_size: str = 'sm',
-                      find_occurances: List[str] = None,
-                      counter: bool = True,
-                      get_linguistic_features: bool = True,
-                      similarity: bool = False,
-                      include_categorical: bool = True,
-                      **kwargs
-                      ) -> TextMiner:
-        """
-        Text analysis
-
-        :param lang: str
-            Pre-defined language
-
-        :param lang_model_size: str
-            Name of the language model size:
-                -> sm: small
-                -> md: mid-size
-                -> lg: large
-
-        :param find_occurances: List[str]
-            Occurances to find (any type)
-
-        :param counter: bool
-            Whether to generate simple counter features
-
-        :param get_linguistic_features: bool
-            Whether to generate and explore linguistic features extracted by processing features containing natural language
-
-        :param similarity: bool
-            Whether to calculate similarity of text or not
-
-        :param include_categorical: bool
-            Include categorical features
-
-        :return: TextMiner object:
-            Results of text mining
-        """
-        _features: List[str] = []
-        if kwargs.get('include') is not None:
-            for include in list(set(kwargs.get('include'))):
-                if include in self.features:
-                    _features.append(include)
-        if kwargs.get('exclude') is not None:
-            _features: List[str] = self.features
-            for exclude in list(set(kwargs.get('exclude'))):
-                if exclude in _features:
-                    del _features[_features.index(exclude)]
-        if len(_features) == 0:
-            _features = self.features
-        _text_features: List[str] = [id_text for id_text in self.feature_types.get('id_text') if id_text in _features]
-        if include_categorical:
-            _text_features = _text_features + [cat for cat in self.feature_types.get('categorical') if cat in _features]
-        _text_miner: TextMiner = TextMiner(df=self.df,
-                                           features=_text_features,
-                                           lang=lang,
-                                           lang_model_size=lang_model_size if lang_model_size in ['sm', 'md', 'lg'] else 'sm',
-                                           auto_interpret_natural_language=False,
-                                           **kwargs
-                                           )
-        if find_occurances is not None:
-            for occurance in find_occurances:
-                _text_miner.count_occurances(features=_text_miner.segments.get('phrases'), search_text=occurance)
-        if counter:
-            _text_miner.count_occurances(features=_text_miner.segments.get('phrases'),
-                                         count_length=True,
-                                         count_numbers=True,
-                                         count_characters=True,
-                                         count_special_characters=True
-                                         )
-        if get_linguistic_features:
-            _text_miner.generate_linguistic_features()
-        if similarity:
-            _text_miner.similarity(features=_text_miner.segments.get('phrases'))
-        _text_miner.get_numeric_features()
-        return _text_miner
