@@ -1,13 +1,25 @@
+"""
+
+Import and export data sets
+
+"""
+
+import boto3
+import io
 import json
 import os
 import pandas as pd
 import pickle
+import pyLDAvis
 import sqlite3
 import zipfile
 
 from dask import dataframe as dd
+from google.cloud import storage
 from sqlalchemy import create_engine
-from typing import List
+from typing import List, Union
+
+CLOUD_PROVIDER: List[str] = ['aws', 'google']
 
 
 class FileUtilsException(Exception):
@@ -19,15 +31,32 @@ class FileUtilsException(Exception):
 
 class FileUtils:
     """
-    Class for handling files
+    Class for handling file utilities
     """
-    def __init__(self, file_path: str, create_dir: bool = True):
+    def __init__(self,
+                 file_path: str,
+                 create_dir: bool = False,
+                 cloud: str = None,
+                 bucket_name: str = None,
+                 region: str = None
+                 ):
         """
         :param file_path: str
             File path
 
-        :param create_dir
-            Boolean indicating whether to create directories if they are not existed
+        :param create_dir: bool
+            Create directories if they are not existed
+
+        :param cloud: str
+            Name of the cloud provider:
+                -> google: Google Cloud Storage
+                -> aws: AWS Cloud
+
+        :param bucket_name: str
+            Name of the bucket of the cloud provider
+
+        :param region: str
+            Name of the region (AWS S3-Bucket)
         """
         if len(file_path) == 0:
             raise FileUtilsException('No file path found')
@@ -35,11 +64,27 @@ class FileUtils:
         self.file_name: str = self.full_path.split('/')[-1]
         self.file_path: str = self.full_path.replace(self.file_name, '')
         _file_type = self.file_name.split('.')
-        if len(_file_type) > 0:
+        if len(_file_type) > 1:
             self.file_type = _file_type[len(_file_type) - 1]
         else:
             self.file_type = None
         self.create_dir = create_dir
+        self.cloud: str = cloud
+        self.bucket_name: str = bucket_name
+        self.region: str = region
+        self.aws_s3_file_name: str = None
+        self.aws_s3_file_path: str = None
+        self.google_cloud_file_name: str = None
+        self.google_cloud_file_path: str = None
+        if self.cloud is not None:
+            if self.cloud not in CLOUD_PROVIDER:
+                raise FileUtilsException('Cloud provider ({}) not supported'.format(self.cloud))
+        if self.cloud == 'aws':
+            self.aws_s3_file_name = '/'.join(self.full_path.split('//')[1].split('/')[1:])
+            self.aws_s3_file_path = '/'.join(self.full_path.split('//')[1].split('/')[1:-1])
+        if self.cloud == 'google':
+            self.google_cloud_file_name = '/'.join(self.full_path.split('//')[1].split('/')[1:])
+            self.google_cloud_file_path = '/'.join(self.full_path.split('//')[1].split('/')[1:-1])
         if self.create_dir:
             if self.full_path.find('/') >= 0:
                 self.make_dir()
@@ -78,8 +123,11 @@ class DataImporter(FileUtils):
                  file_path: str,
                  as_data_frame: bool = True,
                  use_dask: bool = True,
-                 create_dir: bool = True,
+                 create_dir: bool = False,
                  sep: str = ',',
+                 cloud: str = None,
+                 bucket_name: str = None,
+                 region: str = None,
                  **kwargs: dict
                  ):
         """
@@ -98,10 +146,26 @@ class DataImporter(FileUtils):
         :param sep: str
             File separator
 
+        :param cloud: str
+            Name of the cloud provider:
+                -> google: Google Cloud Storage
+                -> aws: AWS Cloud
+
+        :param bucket_name: str
+            Name of the bucket of the cloud provider
+
+        :param region: str
+            Name of the region (AWS S3-Bucket)
+
         :param kwargs: dict
             Additional key word arguments
         """
-        super().__init__(file_path=file_path, create_dir=create_dir)
+        super().__init__(file_path=file_path,
+                         create_dir=create_dir,
+                         cloud=cloud,
+                         bucket_name=bucket_name,
+                         region=region
+                         )
         self.as_df: bool = as_data_frame
         self.use_dask: bool = use_dask
         self.partitions: int = 4 if kwargs.get('npartitions') is None else kwargs.get('npartitions')
@@ -109,12 +173,21 @@ class DataImporter(FileUtils):
         self.kwargs: dict = kwargs
         self._config_args()
 
+    def _aws_s3(self) -> bytes:
+        """
+        Import file from AWS S3 bucket
+
+        return: bytes
+            File object as bytes
+        """
+        _s3_resource = boto3.resource('s3')
+        return _s3_resource.Bucket(self.bucket_name).Object(self.aws_s3_file_name).get()['Body'].read()
+
     def _config_args(self):
         """
         Set configuration setting for the data import as Pandas DataFrame
         """
-        self.kwargs.update({'filepath': self.full_path,
-                            'sep': self.sep,
+        self.kwargs.update({'sep': self.sep,
                             'decimal': '.' if self.kwargs.get('decimal') is None else self.kwargs.get('decimal'),
                             'header': 0 if self.kwargs.get('header') is None else self.kwargs.get('header'),
                             'encoding': 'utf-8' if self.kwargs.get('encoding') is None else self.kwargs.get('encoding'),
@@ -162,7 +235,7 @@ class DataImporter(FileUtils):
                             'memory_map': False if self.kwargs.get('memory_map') is None else self.kwargs.get('memory_map')
                             })
 
-    def _excel_as_df(self):
+    def _excel_as_df(self) -> Union[dd.DataFrame, pd.DataFrame]:
         """
         Import excel file as Pandas DataFrame
 
@@ -170,7 +243,7 @@ class DataImporter(FileUtils):
             Pandas DataFrame containing the content of the html file
         """
         if self.use_dask:
-            return dd.from_pandas(data=pd.read_excel(io=self.kwargs.get('filepath'),
+            return dd.from_pandas(data=pd.read_excel(io=self.full_path,
                                                      sheet_name=self.kwargs.get('sheet_name'),
                                                      header=self.kwargs.get('header'),
                                                      names=self.kwargs.get('names'),
@@ -199,7 +272,7 @@ class DataImporter(FileUtils):
                                   sort=True if self.kwargs.get('sort') is None else self.kwargs.get('sort'),
                                   name=self.kwargs.get('name')
                                   )
-        return pd.read_excel(io=self.kwargs.get('filepath'),
+        return pd.read_excel(io=self.full_path,
                              sheet_name=self.kwargs.get('sheet_name'),
                              header=self.kwargs.get('header'),
                              names=self.kwargs.get('names'),
@@ -242,18 +315,34 @@ class DataImporter(FileUtils):
                              low_memory=self.kwargs.get('low_memory'),
                              )
 
-    def _file(self):
+    def _file(self) -> object:
         """
         Import file
 
         :return: object
             File content
         """
-        with open(file=self.file_path,
-                  mode='r' if self.kwargs.get('mode') is None else self.kwargs.get('mode'),
-                  encoding='utf-8' if self.kwargs.get('encoding') is None else self.kwargs.get('encoding')
-                  ) as file:
-            return file.read()
+        if self.cloud is None:
+            with open(file=self.full_path,
+                      mode='r' if self.kwargs.get('mode') is None else self.kwargs.get('mode'),
+                      encoding='utf-8' if self.kwargs.get('encoding') is None else self.kwargs.get('encoding')
+                      ) as file:
+                return file.read()
+        elif self.cloud == 'aws':
+            return self._aws_s3().decode('utf-8')
+        elif self.cloud == 'google':
+            self._google_cloud_storage()
+            with open(self.google_cloud_file_name.split('/')[-1], 'r') as file:
+                return file.read()
+
+    def _google_cloud_storage(self):
+        """
+        Download files from Google Cloud Storage.
+        """
+        _client = storage.Client()
+        _bucket = _client.get_bucket(bucket_or_name=self.bucket_name)
+        _blob = _bucket.blob(blob_name=self.google_cloud_file_name)
+        _blob.download_to_filename(filename=self.google_cloud_file_name.split('/')[-1])
 
     def _html(self):
         """
@@ -268,8 +357,7 @@ class DataImporter(FileUtils):
         :return: List[pd.DataFrame]
             Contents of the html file as pandas data frames
         """
-        return pd.read_html(io=None,
-                            match=None,
+        return pd.read_html(io=self.full_path,
                             flavor=self.kwargs.get('flavor'),
                             header=self.kwargs.get('header'),
                             index_col=self.kwargs.get('index_col'),
@@ -286,25 +374,32 @@ class DataImporter(FileUtils):
                             displayed_only=self.kwargs.get('displayed_only')
                             )
 
-    def _json(self) -> json.load:
+    def _json(self) -> dict:
         """
         Import json file
 
-        :return: json.load
+        :return: dict
             Content of the json file
         """
-        with open(file=self.file_path,
-                  mode='r' if self.kwargs.get('mode') is None else self.kwargs.get('mode'),
-                  encoding='utf-8' if self.kwargs.get('encoding') is None else self.kwargs.get('encoding')
-                  ) as json_file:
-            return json.load(fp=json_file,
-                             cls=self.kwargs.get('cls'),
-                             object_hook=self.kwargs.get('object_hook'),
-                             parse_float=self.kwargs.get('parse_float'),
-                             parse_int=self.kwargs.get('parse_int'),
-                             parse_constant=self.kwargs.get('parse_constant'),
-                             object_pairs_hook=self.kwargs.get('object_pairs_hook')
-                             )
+        if self.cloud is None:
+            with open(file=self.full_path,
+                      mode='r' if self.kwargs.get('mode') is None else self.kwargs.get('mode'),
+                      encoding='utf-8' if self.kwargs.get('encoding') is None else self.kwargs.get('encoding')
+                      ) as _file:
+                return json.load(fp=_file,
+                                 cls=self.kwargs.get('cls'),
+                                 object_hook=self.kwargs.get('object_hook'),
+                                 parse_float=self.kwargs.get('parse_float'),
+                                 parse_int=self.kwargs.get('parse_int'),
+                                 parse_constant=self.kwargs.get('parse_constant'),
+                                 object_pairs_hook=self.kwargs.get('object_pairs_hook')
+                                 )
+        elif self.cloud == 'aws':
+            return json.loads(s=self._aws_s3())
+        elif self.cloud == 'google':
+            self._google_cloud_storage()
+            with open(self.google_cloud_file_name.split('/')[-1], 'r') as _file:
+                return json.load(fp=_file)
 
     def _json_as_df(self):
         """
@@ -314,7 +409,7 @@ class DataImporter(FileUtils):
             Content of the json file
         """
         if self.use_dask:
-            return dd.read_json(url_path=self.kwargs.get('filepath'),
+            return dd.read_json(url_path=self.full_path,
                                 orient='records' if self.kwargs.get('orient') is None else self.kwargs.get('orient'),
                                 lines=self.kwargs.get('lines'),
                                 storage_options=self.kwargs.get('storage_options'),
@@ -326,7 +421,7 @@ class DataImporter(FileUtils):
                                 meta=self.kwargs.get('meta'),
                                 engine=pd.read_json
                                 )
-        return pd.read_json(path_or_buf=self.kwargs.get('filepath'),
+        return pd.read_json(path_or_buf=self.full_path,
                             orient='records' if self.kwargs.get('orient') is None else self.kwargs.get('orient'),
                             typ='frame',
                             dtype=True if self.kwargs.get('dtype') is None else self.kwargs.get('dtype'),
@@ -342,15 +437,40 @@ class DataImporter(FileUtils):
                             compression=self.kwargs.get('compression')
                             )
 
-    def _pickle(self) -> pickle.load:
+    def _parquet(self):
+        """
+        Import parquet file
+
+        :return dask DataFrame
+        """
+        return dd.read_parquet(path=self.full_path,
+                               columns=None,
+                               filters=self.kwargs.get('filters'),
+                               categories=self.kwargs.get('categories'),
+                               index=self.kwargs.get('index'),
+                               storage_options=self.kwargs.get('storage_options'),
+                               engine='pyarrow',
+                               gather_statistics=self.kwargs.get('gather_statistics'),
+                               split_row_groups=self.kwargs.get('split_row_groups'),
+                               chunksize=self.kwargs.get('chunksize')
+                               )
+
+    def _pickle(self) -> object:
         """
         Import pickle file
 
-        :return: pickle.load
+        :return: object
             Content of pickle file
         """
-        with open(self.file_path, 'rb') as file:
-            return pickle.load(file=file)
+        if self.cloud is None:
+            with open(self.full_path, 'rb') as file:
+                return pickle.load(file=file)
+        elif self.cloud == 'aws':
+            return pickle.loads(self._aws_s3())
+        elif self.cloud == 'google':
+            self._google_cloud_storage()
+            with open(self.google_cloud_file_name.split('/')[-1], 'rb') as file:
+                return pickle.load(file=file)
 
     def _pickle_as_df(self):
         """
@@ -360,7 +480,9 @@ class DataImporter(FileUtils):
             Content of the pickle file
         """
         if self.use_dask:
-            return dd.from_pandas(data=pd.read_pickle(filepath_or_buffer=self.full_path, compression=self.kwargs.get('compression')),
+            return dd.from_pandas(data=pd.read_pickle(filepath_or_buffer=self.full_path,
+                                                      compression=self.kwargs.get('compression')
+                                                      ),
                                   npartitions=self.partitions,
                                   chunksize=self.kwargs.get('chunksize'),
                                   sort=True if self.kwargs.get('sort') is None else self.kwargs.get('sort'),
@@ -370,23 +492,24 @@ class DataImporter(FileUtils):
 
     def _text_as_df(self):
         """
-        Import text file (csv, txt) as Pandas DataFrame
+        Import text file (csv, tsv, txt) as Pandas or dask DataFrame
 
         :return: Pandas DataFrame or dask dataframe
             Content of the text file
         """
         if self.use_dask:
-            return dd.read_csv(urlpath=self.kwargs.get('filepath'),
-                               blocksize='default' if self.kwargs.get('blocksize') is None else self.kwargs.get('blocksize'),
+            return dd.read_csv(urlpath=self.full_path,
+                               blocksize='default' if self.partitions > 1 else self.kwargs.get('blocksize'),
                                lineterminator=self.kwargs.get('lineterminator'),
                                #compression=self.kwargs.get('compression'),
                                sample=256000 if self.kwargs.get('sample') is None else self.kwargs.get('sample'),
                                enforce=False if self.kwargs.get('enforce') is None else self.kwargs.get('enforce'),
                                assume_missing=True if self.kwargs.get('assume_missing') is None else self.kwargs.get('assume_missing'),
                                storage_options=self.kwargs.get('storage_options'),
-                               include_path_column=False if self.kwargs.get('include_path_column') is None else self.kwargs.get('include_path_column')
+                               include_path_column=False if self.kwargs.get('include_path_column') is None else self.kwargs.get('include_path_column'),
+                               dtype=str if self.kwargs.get('dtype') is None else self.kwargs.get('dtype')
                                )
-        return pd.read_csv(filepath_or_buffer=self.kwargs.get('filepath'),
+        return pd.read_csv(filepath_or_buffer=self.full_path,
                            sep=self.kwargs.get('sep'),
                            header=self.kwargs.get('header'),
                            names=self.kwargs.get('names'),
@@ -395,7 +518,7 @@ class DataImporter(FileUtils):
                            squeeze=self.kwargs.get('squeeze'),
                            prefix=self.kwargs.get('prefix'),
                            mangle_dupe_cols=self.kwargs.get('mangle_dup_cols'),
-                           dtype=self.kwargs.get('dtype'),
+                           dtype=str if self.kwargs.get('dtype') is None else self.kwargs.get('dtype'),
                            engine=self.kwargs.get('engine'),
                            converters=self.kwargs.get('converters'),
                            true_values=self.kwargs.get('true_values'),
@@ -443,7 +566,9 @@ class DataImporter(FileUtils):
         :return: object
             File content
         """
-        if self.file_type in ['csv', 'tsv', 'txt']:
+        if self.file_type is None:
+            return self._parquet()
+        elif self.file_type in ['csv', 'tsv', 'txt']:
             return self._text_as_df() if self.as_df else self._file()
         elif self.file_type in ['p', 'pkl', 'pickle']:
             return self._pickle_as_df() if self.as_df else self._pickle()
@@ -468,6 +593,8 @@ class DataImporter(FileUtils):
             finally:
                 _con.con.close()
             return _df
+        elif self.file_type == 'parquet':
+            return self._parquet()
         else:
             raise FileUtilsException('File type ({}) not supported'.format(self.file_type))
 
@@ -507,8 +634,11 @@ class DataExporter(FileUtils):
     def __init__(self,
                  obj,
                  file_path: str,
-                 create_dir: bool = True,
+                 create_dir: bool = False,
                  overwrite: bool = False,
+                 cloud: str = None,
+                 bucket_name: str = None,
+                 region: str = None,
                  **kwargs
                  ):
         """
@@ -524,16 +654,32 @@ class DataExporter(FileUtils):
         :param overwrite: bool
             Whether to overwrite an existing file or not
 
+        :param cloud: str
+            Name of the cloud provider:
+                -> google: Google Cloud Storage
+                -> aws: AWS Cloud
+
+        :param bucket_name: str
+            Name of the bucket of the cloud provider
+
+        :param region: str
+            Name of the region (AWS S3-Bucket)
+
         :param kwargs: dict
             Additional key word arguments
         """
-        super().__init__(file_path=file_path, create_dir=create_dir)
+        super().__init__(file_path=file_path,
+                         create_dir=create_dir,
+                         cloud=cloud,
+                         bucket_name=bucket_name,
+                         region=region
+                         )
         self.obj = obj
         if self.create_dir:
             self.make_dir()
         if not overwrite:
             self._avoid_overwriting()
-        self.user_kwargs = kwargs
+        self.kwargs = kwargs
 
     def _avoid_overwriting(self):
         """
@@ -547,55 +693,239 @@ class DataExporter(FileUtils):
             else:
                 self.full_path = self.full_path.replace('({}).{}'.format(_i - 1, self.file_type), '({}).{}'.format(_i, self.file_type))
 
+    def _aws_s3(self, buffer: io):
+        """
+        Upload files to AWS S3 bucket
+
+        :param buffer: io
+            Object bytes or string
+        """
+        _aws_s3_client = boto3.client('s3', region_name=self.region)
+        _aws_s3_client.put_object(Body=buffer.getvalue(), Bucket=self.bucket_name, Key=self.aws_s3_file_name)
+
     def _html(self):
         """
-        Export data as json file
+        Export data as html file
         """
-        with open(self.full_path, 'w', encoding='utf-8') as file:
-            file.write(self.obj)
+        if self.kwargs.get('topic_clustering') is None:
+            if self.kwargs.get('plotly_offline') is None:
+                if self.cloud is None:
+                    with open(self.full_path, 'w', encoding='utf-8') as _file:
+                        _file.write(self.obj)
+                elif self.cloud == 'aws':
+                    _buffer: io.StringIO = io.StringIO()
+                    _buffer.write(self.obj)
+                    self._aws_s3(buffer=_buffer)
+                elif self.cloud == 'google':
+                    if not os.path.exists(self.google_cloud_file_path):
+                        os.makedirs(name=self.google_cloud_file_path, exist_ok=True)
+                    with open(self.google_cloud_file_name, 'w') as _output:
+                        _output.write(self.obj)
+                    self._google_cloud_storage()
+            else:
+                if self.cloud is None:
+                    with open(self.full_path, 'w', encoding='utf-8') as _file:
+                        _file.write(self.obj.to_html())
+                elif self.cloud == 'aws':
+                    _buffer: io.StringIO = io.StringIO()
+                    _buffer.write(self.obj.to_html())
+                    self._aws_s3(buffer=_buffer)
+                elif self.cloud == 'google':
+                    if not os.path.exists(self.google_cloud_file_path):
+                        os.makedirs(name=self.google_cloud_file_path, exist_ok=True)
+                    with open(self.google_cloud_file_name, 'w') as _output:
+                        _output.write(self.obj.to_html())
+                    self._google_cloud_storage()
+        else:
+            if self.cloud is None:
+                with open(self.full_path, 'w', encoding='utf-8') as _file:
+                    pyLDAvis.save_html(self.obj, _file)
+            elif self.cloud == 'aws':
+                _buffer: io.StringIO = io.StringIO()
+                pyLDAvis.save_html(self.obj, _buffer)
+                self._aws_s3(buffer=_buffer)
+            elif self.cloud == 'google':
+                if not os.path.exists(self.google_cloud_file_path):
+                    os.makedirs(name=self.google_cloud_file_path, exist_ok=True)
+                with open(self.google_cloud_file_name, 'w') as _output:
+                    pyLDAvis.save_html(self.obj, _output)
+                self._google_cloud_storage()
 
     def _gitignore(self):
         """
         Export data as .gitignore file
         """
-        with open(self.file_path, 'w', encoding='utf-8') as file:
-            file.write(self.obj)
+        with open(self.full_path, 'w', encoding='utf-8') as _file:
+            _file.write(self.obj)
+
+    def _google_cloud_storage(self):
+        """
+        Upload files to Google Cloud Storage.
+        """
+        _client = storage.Client()
+        _bucket = _client.get_bucket(bucket_or_name=self.bucket_name)
+        _blob = _bucket.blob(blob_name=self.google_cloud_file_name)
+        _blob.upload_from_filename(filename=self.google_cloud_file_name)
+
+    def _jpg_png(self):
+        """
+        Export data as jpg or jpeg or png file
+        """
+        if self.kwargs.get('plotly_offline') is None:
+            if self.cloud is None:
+                with open(self.full_path, 'wb') as _file:
+                    _file.write(self.obj)
+            elif self.cloud == 'aws':
+                _buffer: io.BytesIO = io.BytesIO()
+                _buffer.write(self.obj)
+                self._aws_s3(buffer=_buffer)
+            elif self.cloud == 'google':
+                if not os.path.exists(self.google_cloud_file_path):
+                    os.makedirs(name=self.google_cloud_file_path, exist_ok=True)
+                with open(self.google_cloud_file_name, 'wb') as _output:
+                    _output.write(self.obj)
+                self._google_cloud_storage()
+        else:
+            if self.cloud is None:
+                with open(self.full_path, 'wb') as _file:
+                    _file.write(self.obj.to_image())
+                #self.obj.write_image(self.full_path)
+            elif self.cloud == 'aws':
+                _buffer: io.BytesIO = io.BytesIO()
+                _buffer.write(self.obj.to_image())
+                self._aws_s3(buffer=_buffer)
+            elif self.cloud == 'google':
+                if not os.path.exists(self.google_cloud_file_path):
+                    os.makedirs(name=self.google_cloud_file_path, exist_ok=True)
+                with open(self.google_cloud_file_name, 'wb') as _output:
+                    _output.write(self.obj.to_image())
+                self._google_cloud_storage()
 
     def _json(self):
         """
         Export data as json file
         """
-        with open(self.full_path, 'w', encoding='utf-8') as file:
-            json.dump(self.obj, file, ensure_ascii=False)
+        if self.cloud is None:
+            with open(self.full_path, 'w', encoding='utf-8') as file:
+                json.dump(obj=self.obj, fp=file, ensure_ascii=False)
+        elif self.cloud == 'aws':
+            _buffer: io.StringIO = io.StringIO()
+            json.dump(obj=self.obj, fp=_buffer, ensure_ascii=False)
+            self._aws_s3(buffer=_buffer)
+        elif self.cloud == 'google':
+            if not os.path.exists(self.google_cloud_file_path):
+                os.makedirs(name=self.google_cloud_file_path, exist_ok=True)
+            with open(self.google_cloud_file_name, 'w') as _output:
+                json.dump(obj=self.obj, fp=_output, ensure_ascii=False)
+            self._google_cloud_storage()
+
+    def _parquet(self):
+        """
+        Export data as parquet file
+        """
+        dd.to_parquet(df=self.obj,
+                      path=self.full_path,
+                      engine='auto',
+                      compression='default' if self.kwargs.get('compression') is None else self.kwargs.get('compression'),
+                      write_index=True if self.kwargs.get('write_index') is None else self.kwargs.get('write_index'),
+                      append=False if self.kwargs.get('append') is None else self.kwargs.get('append'),
+                      ignore_divisions=False if self.kwargs.get('ignore_divisions') is None else self.kwargs.get('ignore_divisions'),
+                      partition_on=self.kwargs.get('partition_on'),
+                      storage_options=self.kwargs.get('storage_options'),
+                      write_metadata_file=True if self.kwargs.get('write_metadata_file') is None else self.kwargs.get('write_metadata_file'),
+                      compute=True if self.kwargs.get('compute') is None else self.kwargs.get('compute'),
+                      compute_kwargs=self.kwargs.get('compute_kwargs'),
+                      schema='infer' if self.kwargs.get('schema') is None else self.kwargs.get('schema')
+                      )
 
     def _pickle(self):
         """
         Export data as pickle file
         """
-        with open(self.full_path, 'wb') as _output:
-            pickle.dump(self.obj, _output, pickle.HIGHEST_PROTOCOL)
+        if self.cloud is None:
+            with open(self.full_path, 'wb') as _output:
+                pickle.dump(self.obj, _output, pickle.HIGHEST_PROTOCOL)
+        elif self.cloud == 'aws':
+            _buffer: io.BytesIO = io.BytesIO()
+            pickle.dump(obj=self.obj, file=_buffer, protocol=pickle.HIGHEST_PROTOCOL)
+            self._aws_s3(buffer=_buffer)
+        elif self.cloud == 'google':
+            if not os.path.exists(self.google_cloud_file_path):
+                os.makedirs(name=self.google_cloud_file_path, exist_ok=True)
+            with open(self.google_cloud_file_name, 'wb') as _output:
+                pickle.dump(obj=self.obj, file=_output, protocol=pickle.HIGHEST_PROTOCOL)
+            self._google_cloud_storage()
 
     def _py(self):
         """
         Export data as python file
         """
-        with open(self.full_path, 'w') as file:
-            file.write(self.obj)
+        if self.cloud is None:
+            with open(self.full_path, 'w') as _file:
+                _file.write(self.obj)
+        elif self.cloud == 'aws':
+            _buffer: io.StringIO = io.StringIO()
+            _buffer.write(self.obj)
+            self._aws_s3(buffer=_buffer)
+        elif self.cloud == 'google':
+            if not os.path.exists(self.google_cloud_file_path):
+                os.makedirs(name=self.google_cloud_file_path, exist_ok=True)
+            with open(self.google_cloud_file_name, 'w') as _output:
+                _output.write(self.obj)
+            self._google_cloud_storage()
 
     def _text(self):
         """
         Export data as text (txt, csv) file
         """
-        _txt = open(self.full_path, 'wb')
-        _txt.write(self.obj)
-        _txt.close()
+        if self.cloud is None:
+            with open(self.full_path, 'w') as _file:
+                _file.write(self.obj)
+        elif self.cloud == 'aws':
+            _buffer: io.StringIO = io.StringIO()
+            _buffer.write(self.obj)
+            self._aws_s3(buffer=_buffer)
+        elif self.cloud == 'google':
+            if not os.path.exists(self.google_cloud_file_path):
+                os.makedirs(name=self.google_cloud_file_path, exist_ok=True)
+            with open(self.google_cloud_file_name, 'w') as _output:
+                _output.write(self.obj)
+            self._google_cloud_storage()
+
+    def _text_from_df(self):
+        """
+        Export data as text file from data frame
+        """
+        self.obj.to_csv(path_or_buf=self.full_path,
+                        sep=self.kwargs.get('sep'),
+                        na_rep="" if self.kwargs.get('na_rep') is None else self.kwargs.get('na_rep'),
+                        float_format=self.kwargs.get('float_format'),
+                        columns=self.kwargs.get('columns'),
+                        header=True if self.kwargs.get('header') is None else self.kwargs.get('header'),
+                        index=False if self.kwargs.get('index') is None else self.kwargs.get('index'),
+                        index_label=self.kwargs.get('index_label'),
+                        mode='w',
+                        encoding=self.kwargs.get('encoding'),
+                        compression='infer' if self.kwargs.get('compression') is None else self.kwargs.get('compression'),
+                        quoting=self.kwargs.get('quoting'),
+                        chunksize=self.kwargs.get('chunksize'),
+                        date_format=self.kwargs.get('date_format'),
+                        doublequote=True if self.kwargs.get('doublequote') is None else self.kwargs.get('doublequote'),
+                        escapechar=self.kwargs.get('escapechar'),
+                        decimal='.' if self.kwargs.get('decimal') is None else self.kwargs.get('decimal')
+                        )
 
     def file(self):
         """
         Export data as file object
         """
-        if self.file_type in ['csv', 'txt']:
-            return self._text()
+        if self.file_type is None:
+            return self._parquet()
+        elif self.file_type in ['csv', 'tsv', 'txt']:
+            if isinstance(self.obj, pd.DataFrame) or isinstance(self.obj, dd.DataFrame):
+                return self._text_from_df()
+            else:
+                return self._text()
         elif self.file_type in ['', 'p', 'pkl', 'pickle']:
             return self._pickle()
         elif self.file_type == 'json':
@@ -604,6 +934,10 @@ class DataExporter(FileUtils):
             return self._py()
         elif self.file_type == 'gitignore':
             return self._gitignore()
+        elif self.file_type == 'parquet':
+            return self._parquet()
+        elif self.file_type in ['jpg', 'jpeg', 'png']:
+            return self._jpg_png()
         else:
             return self._text()
 
